// © 2023 John Breaux
//! An [Identifier] stores the hash of an identifier
use super::*;
use std::rc::Rc;
#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Identifier {
    str: Rc<str>,
}

impl Identifier {
    fn str<T: AsRef<str>>(s: T) -> Self { Self { str: s.as_ref().to_owned().into() } }
}

impl Parsable for Identifier {
    fn parse<'text, T>(_: &Parser, stream: &mut T) -> Result<Self, ParseError>
    where T: TokenStream<'text> {
        let token = stream.expect(Type::Identifier)?;
        match token.variant() {
            Type::Identifier => Ok(Self::str(token.lexeme())),
            _ => unreachable!("Expected identifier, got {token:?}"),
        }
    }
}
impl Display for Identifier {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result { Display::fmt(&self.str, f) }
}
